@@ -31,6 +31,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
+#include <gio/gio.h>
 #include <gtk/gtk.h>
 
 #include <glade/glade-xml.h>
@@ -1414,7 +1415,6 @@ gsm_properties_dialog_dispose (GObject *object)
 
         dialog = GSM_PROPERTIES_DIALOG (object);
 
-        g_debug ("GsmPropertiesDialog: dispose called");
         if (dialog->priv->xml != NULL) {
                 g_object_unref (dialog->priv->xml);
                 dialog->priv->xml = NULL;
@@ -1480,8 +1480,6 @@ gsm_properties_dialog_finalize (GObject *object)
 
         g_return_if_fail (dialog->priv != NULL);
 
-        g_debug ("GsmPropertiesDialog: finalizing");
-
         G_OBJECT_CLASS (gsm_properties_dialog_parent_class)->finalize (object);
 }
 
