@@ -36,6 +36,7 @@
 
 #include "gsm-inhibit-dialog.h"
 #include "gsm-store.h"
+#include "gsm-client.h"
 #include "gsm-inhibitor.h"
 #include "eggdesktopfile.h"
 #include "gsm-util.h"
@@ -66,6 +67,7 @@ struct GsmInhibitDialogPrivate
         int                action;
         gboolean           is_done;
         GsmStore          *inhibitors;
+        GsmStore          *clients;
         GtkListStore      *list_store;
         gboolean           have_xrender;
         int                xrender_event_base;
@@ -76,6 +78,7 @@ enum {
         PROP_0,
         PROP_ACTION,
         PROP_INHIBITOR_STORE,
+        PROP_CLIENT_STORE
 };
 
 enum {
@@ -489,11 +492,15 @@ add_inhibitor (GsmInhibitDialog *dialog,
         GError         *error;
         char          **search_dirs;
         guint           xid;
+        char           *freeme;
 
         /* FIXME: get info from xid */
 
+        desktop_file = NULL;
         name = NULL;
         pixbuf = NULL;
+        freeme = NULL;
+
         app_id = gsm_inhibitor_peek_app_id (inhibitor);
 
         if (IS_STRING_EMPTY (app_id)) {
@@ -553,6 +560,19 @@ add_inhibitor (GsmInhibitDialog *dialog,
                 }
         }
 
+        /* try client info */
+        if (name == NULL) {
+                const char *client_id;
+                client_id = gsm_inhibitor_peek_client_id (inhibitor);
+                if (! IS_STRING_EMPTY (client_id)) {
+                        GsmClient *client;
+                        client = GSM_CLIENT (gsm_store_lookup (dialog->priv->clients, client_id));
+                        if (client != NULL) {
+                                freeme = name = gsm_client_get_app_name (client);
+                        }
+                }
+        }
+
         if (name == NULL) {
                 if (! IS_STRING_EMPTY (app_id)) {
                         name = app_id;
@@ -579,9 +599,13 @@ add_inhibitor (GsmInhibitDialog *dialog,
                                            -1);
 
         g_free (desktop_filename);
+        g_free (freeme);
         if (pixbuf != NULL) {
                 g_object_unref (pixbuf);
         }
+        if (desktop_file != NULL) {
+                egg_desktop_file_free (desktop_file);
+        }
 }
 
 static gboolean
@@ -713,6 +737,23 @@ gsm_inhibit_dialog_set_inhibitor_store (GsmInhibitDialog *dialog,
         }
 }
 
+static void
+gsm_inhibit_dialog_set_client_store (GsmInhibitDialog *dialog,
+                                     GsmStore         *store)
+{
+        g_return_if_fail (GSM_IS_INHIBIT_DIALOG (dialog));
+
+        if (store != NULL) {
+                g_object_ref (store);
+        }
+
+        if (dialog->priv->clients != NULL) {
+                g_object_unref (dialog->priv->clients);
+        }
+
+        dialog->priv->clients = store;
+}
+
 static void
 gsm_inhibit_dialog_set_property (GObject        *object,
                                  guint           prop_id,
@@ -728,6 +769,9 @@ gsm_inhibit_dialog_set_property (GObject        *object,
         case PROP_INHIBITOR_STORE:
                 gsm_inhibit_dialog_set_inhibitor_store (dialog, g_value_get_object (value));
                 break;
+        case PROP_CLIENT_STORE:
+                gsm_inhibit_dialog_set_client_store (dialog, g_value_get_object (value));
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -749,6 +793,9 @@ gsm_inhibit_dialog_get_property (GObject        *object,
         case PROP_INHIBITOR_STORE:
                 g_value_set_object (value, dialog->priv->inhibitors);
                 break;
+        case PROP_CLIENT_STORE:
+                g_value_set_object (value, dialog->priv->clients);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -905,16 +952,19 @@ gsm_inhibit_dialog_constructor (GType                  type,
 #ifdef HAVE_XRENDER
         gdk_error_trap_push ();
         if (XRenderQueryExtension (GDK_DISPLAY (), &dialog->priv->xrender_event_base, &dialog->priv->xrender_error_base)) {
-                g_warning ("GsmInhibitDialog: Initialized XRender extension");
+                g_debug ("GsmInhibitDialog: Initialized XRender extension");
                 dialog->priv->have_xrender = TRUE;
         } else {
-                g_warning ("GsmInhibitDialog: Unable to initialize XRender extension");
+                g_debug ("GsmInhibitDialog: Unable to initialize XRender extension");
                 dialog->priv->have_xrender = FALSE;
         }
         gdk_display_sync (gdk_display_get_default ());
         gdk_error_trap_pop ();
 #endif /* HAVE_XRENDER */
 
+        /* FIXME: turn this on when it is ready */
+        dialog->priv->have_xrender = FALSE;
+
         setup_dialog (dialog);
 
         gtk_widget_show_all (GTK_WIDGET (dialog));
@@ -934,20 +984,19 @@ gsm_inhibit_dialog_dispose (GObject *object)
 
         g_debug ("GsmInhibitDialog: dispose called");
 
-        g_signal_handlers_disconnect_by_func (dialog->priv->inhibitors,
-                                              on_store_inhibitor_added,
-                                              dialog);
-        g_signal_handlers_disconnect_by_func (dialog->priv->inhibitors,
-                                              on_store_inhibitor_removed,
-                                              dialog);
-
         if (dialog->priv->list_store != NULL) {
                 g_object_unref (dialog->priv->list_store);
                 dialog->priv->list_store = NULL;
         }
 
-
         if (dialog->priv->inhibitors != NULL) {
+                g_signal_handlers_disconnect_by_func (dialog->priv->inhibitors,
+                                                      on_store_inhibitor_added,
+                                                      dialog);
+                g_signal_handlers_disconnect_by_func (dialog->priv->inhibitors,
+                                                      on_store_inhibitor_removed,
+                                                      dialog);
+
                 g_object_unref (dialog->priv->inhibitors);
                 dialog->priv->inhibitors = NULL;
         }
@@ -982,6 +1031,13 @@ gsm_inhibit_dialog_class_init (GsmInhibitDialogClass *klass)
                                                               NULL,
                                                               GSM_TYPE_STORE,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+        g_object_class_install_property (object_class,
+                                         PROP_CLIENT_STORE,
+                                         g_param_spec_object ("client-store",
+                                                              NULL,
+                                                              NULL,
+                                                              GSM_TYPE_STORE,
+                                                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
         g_type_class_add_private (klass, sizeof (GsmInhibitDialogPrivate));
 }
@@ -1031,6 +1087,7 @@ gsm_inhibit_dialog_finalize (GObject *object)
 
 GtkWidget *
 gsm_inhibit_dialog_new (GsmStore *inhibitors,
+                        GsmStore *clients,
                         int       action)
 {
         GObject *object;
@@ -1038,6 +1095,7 @@ gsm_inhibit_dialog_new (GsmStore *inhibitors,
         object = g_object_new (GSM_TYPE_INHIBIT_DIALOG,
                                "action", action,
                                "inhibitor-store", inhibitors,
+                               "client-store", clients,
                                NULL);
 
         return GTK_WIDGET (object);
